{
    "description": "阿里云盘 WebDAV 服务器",
    "home_url": "Module_aliyundrivewebdav.asp",
    "md5": "c0bac93db83f87293a1e4efb7aa5a97c",
    "module": "aliyundrivewebdav",
    "title": "阿里云盘 WebDAV",
    "version": "0.4.2"
}
